import subprocess
import threading

from gi.repository import GLib

class CommandRunner:
    def __init__(self, on_done=None):
        self.dialog = None
        self.textbuffer = None

        self.on_done = on_done  # Callback function to be called after completion

    def run_command(self, command, dialog):
        self.dialog = dialog
        self.textbuffer = dialog.get_child().get_child().get_child().get_buffer()

        def append_log(text):
            GLib.idle_add(self.textbuffer.insert_at_cursor, text)
            GLib.idle_add(dialog.get_child().get_child().get_child().scroll_to_mark,
                          self.textbuffer.get_insert(), 0, False, 0.0, 1.0)

        def read_output(source):
            while True:
                line = source.readline()
                if line:
                    GLib.idle_add(append_log, line)
                else:
                    break

        def process_runner():
            try:
                process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
                                           stderr=subprocess.PIPE, text=True)
                threading.Thread(target=read_output, daemon=True, args=(process.stdout,)).start()
                threading.Thread(target=read_output, daemon=True, args=(process.stderr,)).start()

                process.wait()

                GLib.idle_add(dialog.force_close)

                if self.on_done:
                    GLib.idle_add(self.on_done)  # Call the callback function after completion

            except subprocess.CalledProcessError as e:
                GLib.idle_add(append_log, f"Ошибка: {str(e)}\n")
                GLib.idle_add(dialog.force_close)
                if self.on_done:
                    GLib.idle_add(self.on_done)  # Call the callback function after completion

        threading.Thread(target=process_runner, daemon=True).start()
